package net.sourceforge.cathcart;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import net.sourceforge.cathcart.typedefs.PropertyType;

import org.apache.tools.ant.BuildException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Parses an xml configuration file for information on valid tasks and
 * properties.
 * 
 * @author Steven Hale
 * 
 */
public class TaskParser {
	private static final String AND = " and ";
	private final InputStream stream;
	private final DocumentBuilderFactory factory;
	public final static String URL = "url";

	public TaskParser(InputStream stream, DocumentBuilderFactory factory) {
		this.stream = stream;
		this.factory = factory;
	}

	public Document parse() {
		Document dom = null;
		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
			dom = builder.parse(stream);
		} catch (ParserConfigurationException e) {
			throw new RuntimeException(e);
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}

		return dom;
	}

	public List<Element> getTools(Document doc) {
		Element root = doc.getDocumentElement();
		List<Element> toolsList = new ArrayList<Element>();
		NodeList tools = root.getElementsByTagName("tool");
		for (int j = 0; j < tools.getLength(); j++) {
			NodeList nodes = tools.item(j).getChildNodes();

			for (int i = 0; i < nodes.getLength(); i++) {
				try {
					toolsList.add((Element) nodes.item(i));
				} catch (ClassCastException e) {// NOPMD
				}

			}
		}
		return toolsList;
	}

	public Element getTool(String toolName, Document doc) {
		Element root = doc.getDocumentElement();
		NodeList toolNodes = root.getElementsByTagName(toolName);
		Element tool = (Element) toolNodes.item(0);
		if (tool != null) {
			return tool;
		}
		throw new BuildException(toolName + " not configured");
	}

	public String getXsl(String toolName) {
		return "cathcart_" + toolName + ".xsl";
	}

	protected Map<String, PropertyType> getPropertyTypes(Element tool) {
		Map<String, PropertyType> allowedPropertyTypes = new HashMap<String, PropertyType>();

		NodeList nestedProperties = tool
				.getElementsByTagName("nestedproperties");
		for (int i = 0; i < nestedProperties.getLength(); i++) {
			NodeList propertyTypes = nestedProperties.item(i).getChildNodes();
			for (int j = 0; j < propertyTypes.getLength(); j++) {
				Object object = propertyTypes.item(j);
				try {
					Element element = (Element) object;
					PropertyType propertyType = new PropertyType();
					propertyType.setName(element.getNodeName());
					propertyType.setDesiredTrend(element.getAttribute("trend"));
					allowedPropertyTypes.put(element.getNodeName(),
							propertyType);
				} catch (ClassCastException e) {// NOPMD
				}

			}
		}
		return allowedPropertyTypes;

	}

	public StringBuffer printList(Collection<String> items) {
		StringBuffer sb = new StringBuffer();

		int size = items.size();
		Iterator<String> iterator = items.iterator();
		if (size == 1) {
			sb.append(iterator.next());
			return sb;
		}

		if (size == 2) {
			return printTwoItemList(sb, iterator);
		}

		return printMultiList(items, sb, size);
	}

	private StringBuffer printMultiList(Collection<String> items,
			StringBuffer sb, int size) {
		int i = 0;
		for (String item : items) {
			sb.append(item);
			if (i < size - 2) {
				sb.append(", ");
			} else  if (i == size - 2){
				sb.append(AND);
			}
			i++;
		}
		return sb;
	}

	private StringBuffer printTwoItemList(StringBuffer sb,
			Iterator<String> iterator) {
		sb.append(iterator.next());
		sb.append(AND);
		sb.append(iterator.next());
		return sb;
	}

	public String print(Document doc) {
		StringBuffer buffer = new StringBuffer(100);
		List<Element> tools = this.getTools(doc);
		for (Element t : tools) {
			String toolName = t.getNodeName();
			buffer.append("<checkprevious tool='" + toolName + "'>\n");
			Map<String, PropertyType> pts = getPropertyTypes(doc, toolName);
			Collection<PropertyType> values = pts.values();
			for (PropertyType v : values) {
				buffer.append("\t<propertytype name='" + v.getName()
						+ "' trend='" + v.getDesiredTrend() + "'/>\n");
			}
			buffer.append("</checkprevious>\n\n");
		}
		return buffer.toString();
	}

	public Map<String, PropertyType> getPropertyTypes(Document doc,
			String toolName) {

		Element tool = getTool(toolName, doc);
		return getPropertyTypes(tool);
	}

}
