package net.sourceforge.cathcart.taskdefs;

import java.io.File;

import net.sourceforge.cathcart.typedefs.PropertyType;
import net.sourceforge.cathcart.typedefs.SizeComparison;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.XSLTProcess;
import org.apache.tools.ant.taskdefs.XmlProperty;
import org.apache.tools.ant.util.FileUtils;

/**
 * Checks a value against a predefined limit.
 * 
 * @author Steven Hale
 * 
 */
@SuppressWarnings("deprecation")
public final class Check extends CheckBase {

	private XSLTProcess xsltProcess = new XSLTProcess();

	private XmlProperty xmlProperty = new XmlProperty();

	private File out = FileUtils.getFileUtils().createTempFile("CATHCART", ".xml",
			null, true);

	public void execute() {
		parseConfig();

		logger.log("Temporary file " + out.getAbsolutePath(),
				Project.MSG_VERBOSE);

		fixPropertyTypes();

		
		performXslt(xsltProcess, out);
		performXmlProperty(xmlProperty, out, "");

		handleNestedElements();

	}

	private void handleNestedElements() {
		for (PropertyType pt : propertyTypes) {
			if (pt.isAlwaysSuccessful()) {
				success();
			} else {

				checkNestedElement(pt, true);
				String property = getProject().getProperty(
						tool + "." + pt.getName());
				logger.log(pt.getName() + " " + property, Project.MSG_VERBOSE);

				Double limit = pt.getLimit();

				SizeComparison sizeComparison = chooseComparison(pt);
				setComparisonArguments(pt, property, limit, sizeComparison);
				if (sizeComparison.eval()) {
					failure(pt, sizeComparison);

				} else {
					success(pt, sizeComparison);
				}
			}

		}
	}

	public void setXmlProperty(XmlProperty xmlProperty) {
		this.xmlProperty = xmlProperty;
	}

	public void setXsltProcess(XSLTProcess xsltProcess) {
		this.xsltProcess = xsltProcess;
	}

	public void setOut(File out) {
		this.out = out;
	}

}
