package net.sourceforge.cathcart.typedefs;

import org.apache.tools.ant.BuildException;
/**
 * Compares the size of two operands when trend is expected to be downward.
 * @author Steven Hale
 *
 */
public class DownwardComparison extends SizeComparison {
    
	public boolean eval() throws BuildException {
		return (operandOne > limit());
	}

	protected double limit() {
		double leeway = this.leeway;
		if (absolute) {
			return operandTwo + leeway;
		}
		
		return operandTwo * (1.0 + leeway / 100);
	}

	@Override
	public String toString() {
		String out = "Checking that " + format.format(operandOne) + " <= " + format.format(operandTwo);
		String sLeeway;
		if (leeway >= 0) {
			sLeeway = " + " + format.format(leeway);
		} else {
			sLeeway = " - " + format.format(-leeway);
		}

		if (absolute) {
			return out + sLeeway + " ("
			+ format.format(limit()) + ")";
		}
		
		return out + sLeeway + " %" + " ("
		+ format.format(limit()) + ")";

	}

	@Override
	public String failedMessage() {
		String sLeeway;
		if (leeway >= 0) {
			sLeeway = " + " + format.format(leeway);

		} else {
			sLeeway = " - " + format.format(-leeway);
		}
		if (!absolute) {
			sLeeway += " %";
		}
		return "Expected value to be less than " +format.format(limit())  + " ("
				+ format.format(operandTwo) + sLeeway + ") but was " + format.format(operandOne);
	}

}
